########################################################################################################################
#
# Module: coordonnées des sommets  du parcours et vitesses entre les noeuds
#
# coord_point est un dictionnaire  : coord_point['Pi'] = [xi,yi]
# vit_parcours est un dictionnaire : vit_parcours['Pa0'] = {'Pa1': vit_a01, 'Pa2': vit_ao2, ...}
# fonctions :
# dist(PX, PY) :  calcule la distance en mm entre entre deux noeuds
# vitesse(PX, PY) : renvoie la vitesse définie entre deux noeuds
# calc_trajet(parcours) : donne les déplacements élémentaires en vitesse, ligne et rotation du robot
#
########################################################################################################################




from math import sqrt, atan2, degrees

###################################################################################
## Les coordonnées des points P du plan (abscisse, ordonnée) en mm  sur la carte ##
###################################################################################
coord_point = dict()  # dictionnaire
coord_point['A'] = [250, 165]
coord_point['B'] = [600, 160]
coord_point['C'] = [1375, 165]
coord_point['D'] = [1875, 160]
coord_point['E'] = [2393, 162]
coord_point['F'] = [2607, 619]
coord_point['G'] = [1990, 630]
coord_point['H'] = [1780, 470]
coord_point['I'] = [1420, 632]
coord_point['J'] = [1180, 340]
coord_point['K'] = [760, 650]
coord_point['L'] = [378, 452]
coord_point['M'] = [135, 775]
coord_point['N'] = [330, 930]
coord_point['O'] = [530, 790]
coord_point['P'] = [1020, 840]
coord_point['Q'] = [1370, 1110]
coord_point['R'] = [1965, 880]
coord_point['S'] = [2480, 898]
coord_point['T'] = [580, 1100]
coord_point['Z'] = [800, 800]  # point isolé non joignable

######################################################
## Les vitesses définies des arc  de la carte       ##
######################################################
'''Vitesse min 200, vitesse moyenne 350, vitesse max 500'''

vit_defaut = 350 # vitesse  de déplacement par défaut du robot
# une vitesse peut etre nulle
vit_parcours = dict() # dictionnaire
vit_parcours['A'] = {'B': vit_defaut, 'L': vit_defaut}
vit_parcours['B'] = {'A': vit_defaut, 'C': 180, 'K': vit_defaut}
vit_parcours['C'] = {'B': vit_defaut, 'D': vit_defaut, 'H': vit_defaut, 'J': vit_defaut}
vit_parcours['D'] = {'C': vit_defaut, 'E': vit_defaut, 'H': vit_defaut}
vit_parcours['E'] = {'D': vit_defaut, 'F': vit_defaut}
vit_parcours['F'] = {'E': vit_defaut, 'G': vit_defaut, 'S': vit_defaut}
vit_parcours['G'] = {'H': vit_defaut, 'I': vit_defaut, 'F': vit_defaut, 'R': vit_defaut}
vit_parcours['H'] = {'C': vit_defaut, 'D': vit_defaut, 'G': vit_defaut}
vit_parcours['I'] = {'G': vit_defaut, 'J': vit_defaut, 'Q': vit_defaut}
vit_parcours['J'] = {'C': vit_defaut, 'I': vit_defaut, 'K': vit_defaut}
vit_parcours['K'] = {'B': vit_defaut, 'J': vit_defaut, 'O': vit_defaut, 'P': vit_defaut}
vit_parcours['L'] = {'A': vit_defaut, 'M': vit_defaut, 'O': vit_defaut}
vit_parcours['M'] = {'L': vit_defaut, 'N': vit_defaut}
vit_parcours['N'] = {'M': vit_defaut, 'O': vit_defaut, 'T': vit_defaut}
vit_parcours['O'] = {'K': vit_defaut, 'L': vit_defaut, 'N': vit_defaut}
vit_parcours['P'] = {'K': vit_defaut, 'Q': vit_defaut, 'T': vit_defaut}
vit_parcours['Q'] = {'I': vit_defaut, 'P': vit_defaut, 'R': vit_defaut}
vit_parcours['R'] = {'G': vit_defaut, 'Q': vit_defaut, 'S': vit_defaut}
vit_parcours['S'] = {'F': vit_defaut, 'R': vit_defaut}
vit_parcours['T'] = {'N': vit_defaut, 'P': vit_defaut}


def dist(PX, PY):

    """Cette fonction calcule la longueur des arcs entre deux noeuds adjacents PX et PY,
       à partir du dictionnaire coord_point de la carte """
    global coord_point
    calcul_dist = round(
        sqrt((coord_point[PX][0] - coord_point[PY][0]) ** 2 + (coord_point[PX][1] - coord_point[PY][1]) ** 2), 1)
    return calcul_dist


def vitesse(PX, PY):
    """ Cette fonction  renvoie la vitesse entre  deux noeuds adjacents PX et PY,
        à partir du dictionnaire  vit_parcours de la carte """
    return vit_parcours[PX][PY]












def calc_trajet(parcours):

    """Cette fonction retourne les déplacements élémentaires du robot, en vitesse, ligne  et rotation à partir
    de la liste des  points du parcours  :
    Trois listes son rendues : vit_segement, depl_segment et rot_robot pour les vitesses, les mouvements en ligne droite
    et les rotations (mm et degré).
    """
    # ligne de test de forcage de parcours à commenterv:
    #parcours = ['A','L','M','N','O','K','P','Q','I','J','C','D','E','F','S','R','G','I','Q','P','K','O','L','A']
    # vitesse du robot au début de chaque ligne
    vit_segement = []
    for cpt in range(len(parcours) - 1):
        vit_segement.append(vitesse(parcours[cpt], parcours[cpt + 1]))
    vit_segement.append(0)  # la derniere vitesse est nulle
    print('vitesse du robot : ', vit_segement)

    # déplacements en segement de ligne :
    depl_segment = []
    for cpt in range(len(parcours) - 1):
        depl_segment.append(dist(parcours[cpt], parcours[cpt + 1]))
    depl_segment.append(0)  # le dernier déplacement est nul
    print('deplacement en mm point à point : ', depl_segment)

    # rotations du robot au changement de route :
    angle_segement = []  # liste des angles en Pi des segments (Pi+1, Pi) par rapport horizontale

    for cpt in range(len(parcours) - 1):
        pt1 = parcours[cpt];
        pt2 = parcours[cpt + 1]  # point Pi et Pi+1 du parcours

        xx = coord_point[pt2][0] - coord_point[pt1][0]  # x2 -x1
        yy = coord_point[pt2][1] - coord_point[pt1][1]  # y2 -y1

        angle_segement.append(round((degrees(atan2(yy, xx)))))

    # Référence de rotation : Axe horizontal du tableau en direction de E

    rot_robot = [ angle_segement[0]]  # Aligné sur la référence au départ
    #  calcul angle rotation  (précédent - suivant) et mise au format 180,-180
    for cpt in range(len(parcours) - 2):
        var_angle = angle_segement[cpt + 1] - angle_segement[cpt]
        if var_angle > 180:
            var_angle = var_angle - 360
        if var_angle < -180:
            var_angle = 360 + var_angle
        rot_robot.append(var_angle)

    rot_robot.append(0)  # pas de rotation à l'arrivée
    print('rotation du robot en degré à chaque point : ', rot_robot)


    return vit_segement,depl_segment, rot_robot
