########################################################################################################################
#
# Projet : Robot Rover
# Module principal : communication avec le robot par commandes élémentaires envoyées via le port COM série du PC
# connecté à un module Bluetooth HC05
#
# fonctions :
# portcom.Affich_PortCom(): liste les ports COMX accéssibles
#
########################################################################################################################
'''
Détail des commandes possibles :
 p      : envoi d'un parcours programmé
 w      : stop robot immédiat / initialise le compteur de déplacement
 lxxxx  : ligne de xxxx longueur (24 = un tour de roue soit 13.5 cm, 180 pour la ligne)
 chxxxx : rotation sens horaire de xxxx longueur (51 = un tour complet sur lui même)
 caxxxx : rotation sens anti-horaire de xxxx longueur (51 = un tour complet sur lui même)
 vxxx   : vitesse de base du robot, par défaut 180)
 kxxx   : gain proportionnel du robot, par défaut 15)
 d      : voir le capteur barrette LED
 b      : voir la mesure batterie
 x      : commande de test--> avance d'un tour de roue
 u      : commande de test--> un tour sur lui même
 exemples :l0025,ch9000, d,v300,k005
 q      : pour quitter
 '''


import serial  # pyserial à importer dans paramètres - setting - python interpreter +
import time
import winsound
import portcom
from envoi_robot import execution, read_serial

# Pacours programmé :

def exec(vit,depl,rot) :
    """Cette fonction envoie une commande combineé au robot : choix p  du menu
       programmé
       vit : vitesse min 200, vitesse moyenne 350, vitesse max 500
       depl : déplacement en ligne droite en mm
       rot : rotation en degré signée
    """
    serialPort.close()  # ferme car déjà ouvert
    vit_segment = [vit]
    depl_segment = [depl]
    rot_robot = [rot]
    execution(vit_segment, depl_segment, rot_robot)
    serialPort.open()





def retour_acquitement() :

    """Cette fonction attend une réponse du robot à des commandes en Bluetooth élémentaires, hors modes parcours
    programmé """
    while True:

        reading = serialPort.readline()
        if (str(reading, 'utf-8')) == 'ok\r\n':

            print('Bien reçu')
            break

        elif (str(reading, 'utf-8')) == 'ko\r\n':

            winsound.Beep(840, 50)  # beep : frequence, durée ms
            print('Erreur commande en réception')
            break

        elif (str(reading, 'utf-8')[0:5]) == 'BAR :':
            print(str(reading, 'utf-8'))
            break

        elif (str(reading, 'utf-8')[0:5]) == 'BAT :':
            print(str(reading, 'utf-8'))
            break

        elif (time.time() > timeout):  # time out--> attente réponse non blocante
            winsound.Beep(600, 50)  # beep d'erreur : frequence, durée ms
            print('Pas de réponse')
            break


# Recherche des ports COMX :
com_list = []  # liste port com
com_list = portcom.Affich_PortCom()  # recherche du port Com série
if len(com_list) == 1:
    portcom.com_name = com_list[0]
    print('port com retenu', portcom.com_name)
elif len(com_list) > 1:
    n = input('Taper le numéro du port : ')
    print(n)
    portcom.com_name = com_list[int(n) - 1]

else:
    quit()


serialPort = serial.Serial(portcom.com_name, 115200, timeout=3)  # timeout en seconde

# serialPort.open()


# Au départ une vitesse par défaut est envoyée au robot dans la cas ou sa vitesse serait trop faible ou nulle
serialPort.write(bytes('  ' + 'v180' + '\r\n', 'UTF-8'))
retour_acquitement()
serialPort.close()

# Menu
print('Ports Comx : ', portcom.com_name)
print(' Commandes : ')
print(" p      : envoi d'un parcours programmé")
print(' w      : stop robot immédiat/ initialise le compteur de déplacement')
print(' lxxxx  : ligne de xxxx longueur (24 = un tour de roue soit 13.5 cm)')
print(' chxxxx : rotation sens horaire de xxxx longueur (51 = un tour complet sur lui même)')
print(' caxxxx : rotation sens anti-horaire de xxxx longueur (51 = un tour complet sur lui même)')
print(' vxxx   : vitesse de base du robot, par défaut 180, max 500)')
print(' kxxx   : gain proportionnel du robot, par défaut 15)')
print(' d      : voir le capteur barrette LED')
print(' b      : voir la mesure batterie ')
print(' x      : commande de test--> avance d''un tour de roue ')
print(' u      : commande de test--> un tour sur lui même ')
print(' q      : pour quitter')


while True:
    print('')
    choix = input('Taper la commande : ')
    if choix == 'q':
        quit()

    if choix == '':
        choix = ' '
    if choix[0] == 'x':
        choix = 'l0024'
    if choix[0] == 'u':
        choix = 'ch0051'
# Envoi des commandes combinées, préenregitrées  :
    if choix == 'p':

        # exec(vitesse,longueur de déplacement en mm, angle de rotation en degré)
        serialPort.open()
        exec(180, 360, 0)   # avance de 2 m pour tester sur la ligne courbe à la vitesse 180....

        ##################################################################
        # COMPLETER ICI pour faire le test de la trajectoire
        ##################################################################
        #
        #

    else :
        serialPort.open()  # ouverture port
        commande = '  ' + choix + '\r\n'  # 2 caractères vide pour synchro
        print('commande :', commande)
        serialPort.write(bytes(commande, 'UTF-8'))
        timeout = time.time() + 4  # 4 secondes à partir de maintenant pour recevoir une réponse
        # Boucle d'attente d'un retour du robot :
        retour_acquitement()
    serialPort.close() # fermeture port