########################################################################################################################
#
# Module: communication Bluetooth vers le robot via un port COM série ouvert sur le PC
#
# fonctions :
# read_serial(serialPort,cmd_en_cours,commande) : acquittement d'un déplacement en ligne ou rotation par le robot
#
# execution(vit_segment,depl_segment ,rot_robot ) : envoi des commandes de mouvement élémentaire au robot
#
#
########################################################################################################################

import serial
import time
import portcom




#
def read_serial(serialPort,cmd_en_cours,commande,acquitement):

    """Cette fonction attend le retour d'exécution du robot par Bluetooth :
        à chaque commande élémentaire est associée un retoure d'acquitement du robot :
        'ok\r\n' pour l'envoie de la vitesse
        'complete\r\n' pour la rotation et la ligne (en fait un ok arrive aussi  pour dire que la commande reçue est valable
        mais on attendra complete, qui dit que la commande est réalisée par une rotation ou un déplacement)
    """
    while True:

        reading = serialPort.readline()

        if (str(reading, 'utf-8')) == acquitement:  # réception binaire vers ascii
            print(cmd_en_cours, '  :')
            print(' --> ', commande[:-2], 'traitée\r\n')

            break






def execution(vit_segment,depl_segment ,rot_robot ) :

    """Cette fonction envoie les commandes de déplacement élémentaire au robot, en Bluetooth
    A partir des 3 listes en entrée:
    vit_segment : vitesse0, vitesse1, vitesse2 ... 0
    rot_robot  : rotation0, rotation1, rotation2 ... 0
    depl_segment : ligne0, ligne1, ligne2 ... 0
    vitesse, rotation et ligne sont à zéro point d'arrivée

    La séquence envoyée dans l'ordre et en série par Bluetooth est :
    vitesse0, rotation0, ligne0,   vitesse1, rotation1, ligne1,   vitesse2, rotation2, ligne2  ... 0, 0, 0
    Du point de départ au point d'arrivée ou les commandes sont à zéro (pas d'action)
    A chaque commande élémentaire est associée un acquitement envoyé par le robot 'ok\r\n' ou 'complete\r\n'
    """

    serialPort = serial.Serial(portcom.com_name , 115200, timeout=4)  # timeout en seconde
    serialPort.close()
    serialPort.open()

    for k in range(len(rot_robot)):

        # Les vitesses
        cmd_en_cours = str(vit_segment[k]) + '  speed'
        acquitement = 'ok\r\n'
        send = 'v' + ('000' + str(vit_segment[k]))[-3:]
        #send = 'v' + ('000' + str(200))[-3:]
        commande = '  ' + send + '\r\n'  # 2 caractères vide pour synchro pas obligatoire
        # # envoi commande
        serialPort.write(bytes(commande, 'UTF-8'))  # ascii vers binaire 8 bits
        # attente du retour acquitement vitesse reçue: 'ok'
        read_serial(serialPort, cmd_en_cours, commande, acquitement)

        # Les rotations :
        cmd_en_cours = str(rot_robot[k]) + '°'
        # résolution environ +-7°
        cmd_en_cours = str(rot_robot[k]) + '°'
        rot = round(rot_robot[k] * 51 / 360)
        if rot >= 0:
            send = 'ca' + ('0000' + str(rot))[-4:]

        elif rot < 0:
            send = 'ch' + ('0000' + str(abs(rot)))[-4:]

        commande = '  ' + send + '\r\n'  # 2 caractères vide pour synchro
        # envoi commande
        serialPort.write(bytes(commande, 'UTF-8'))
        # attente du retour de la réalisation de la rotation par le robot : 'complete'
        acquitement ='complete\r\n'
        read_serial(serialPort,cmd_en_cours,commande,acquitement)
        #

        # Les lignes :
        # résolution environ  0.6 cm (14 /24)
        cmd_en_cours = str(depl_segment[k]) + 'mm'
        lig = round(0.0 + depl_segment[k] * 1.00 * 24 / (13.5 * 10))
        send = 'l' + ('0000' + str(lig))[-4:]
        commande = '  ' + send + '\r\n'  # 2 caractères vide pour synchro pas obligatoire
        # # envoi commande
        serialPort.write(bytes(commande, 'UTF-8'))  # ascii vers binaire 8 bits
        # attente du retour de la réalisation de la ligne par le robot : 'complete'
        acquitement = 'complete\r\n'
        read_serial(serialPort,cmd_en_cours,commande,acquitement)
        #


        #
        time.sleep(1)


    print('...................Tâche terminée')



